#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXNODE 1000000
#define MAXPOW	1000000000
#define BUFLEN 1024

int capacity[MAXNODE+1] = {0};
int power[MAXNODE+1] = {0};
int pp[MAXNODE+1] = {0};
int parent[MAXNODE+1] = {0};
int source[MAXNODE+1] = {0};
int ss[MAXNODE+1] = {0};

char buf[BUFLEN];

int N;
int impossible = 0;

#define WA return 0
#define OK return 1

// Returns 1 for OK, 0 for WA
int check(FILE* in, FILE* out, FILE* notes) {
  fgets(buf, BUFLEN-1, in);
  sscanf(buf, "%d%d", &N, &power[1]);
  power[1] = -power[1]; // if town, the power is negative

  // read the input
  for (int i=2;i<=N;i++) {
    char type;
    fgets(buf, BUFLEN-1, in);
    sscanf(buf, "%c%d%d%d", &type, &parent[i], &capacity[i], &power[i]);

    if (type == 'M') // town
      power[i] = -power[i];

    if (type == 'E' && power[i] == 0) { // power station with no production, mark this specially
      capacity[i] = -1;
      parent[i] = 0;
    }

    pp[i] = power[i];
  }

  // read the output
  if (fscanf(out, "Nelz%c", &buf[0]) > 0) {
    if (buf[0] == 'e')
      impossible = 1;
    else
      WA;
  } else
    for (int i=1;i<=N;i++) {
      if (fscanf(out, "%d", &source[i]) <= 0)
	WA;

      if (source[i] > N || source[i] < 1)
	WA;
    }

  // read the notes
  fgets(buf, BUFLEN-1, notes);
  if (buf[0] == 'N') {
    if (impossible)
      OK;
    else
      WA;
  }

  // fold the tree
  for (int i=N;i>1;i--) {
    if (source[i] != i) { // not source for myself
      if (pp[i] > 0) // power station
	WA;

      if (ss[i]) { // connected to source
	if (source[parent[i]] == source[i]) { // supplying also to the parent
	  if (capacity[i] < power[i]) // the uplink capacity is limiting
	    power[i] = capacity[i];

	  if (power[i] + power[parent[i]] < 0) // this town cannot power the parent
	    WA;

	  power[parent[i]] += power[i]; // adjust the parent
	  ss[parent[i]] = 1; // and is also connected
	} else {
	  if (power[i] < 0) // this town is underpowered
	    WA;
	}
      } else { // not connected to source
	if (source[parent[i]] == source[i]) { // supplied from the parent
	  if (capacity[i] < -power[i]) // unsufficient capacity
	    WA;

	  power[parent[i]] += power[i]; // adjust the parent
	} else {
	  if (power[i] < 0) // this town is underpowered
	    WA;
	}
      }

    } else if (source[i] == i) { // being the source
      if (pp[i] < 0) // town
	WA;

      if (power[i] < 0) // exhausted
	WA;

      if (source[parent[i]] == i) { // source for the parent
	power[parent[i]] += power[i];
	ss[parent[i]] = 1;
      }
    }
  }

  if (power[1] >= 0 && source[1] != 1)
    OK;

  WA;
}

int main(int argc, char ** argv) {
  FILE* fp_in = fopen(argv[1], "r");
  FILE* fp_out = fopen(argv[2], "r");
  FILE* fp_notes = fopen(argv[3], "r");
  if (!fp_in || !fp_out || !fp_notes)
    return 2;

  return !check(fp_in, fp_out, fp_notes);
}
