#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#define MAXNODE 1000000
#define MAXPOW	1000000000
#define BUFLEN 1024

int capacity[MAXNODE+1] = {0};
int power[MAXNODE+1] = {0};
int parent[MAXNODE+1] = {0};
int newpar[MAXNODE+1] = {0};
int newpow[MAXNODE+1] = {0};

char buf[BUFLEN];

int N;

void fail(void) {
  printf("Nelze\n");
  exit(0);
}

int findpst(int i);

int main(void) {
  fgets(buf, BUFLEN-1, stdin);
  sscanf(buf, "%d%d", &N, &power[1]);
  newpow[1] = power[1] = -power[1]; // if town, the power is negative

  // read the input
  for (int i=2;i<=N;i++) {
    char type;
    fgets(buf, BUFLEN-1, stdin);
    sscanf(buf, "%c%d%d%d", &type, &parent[i], &capacity[i], &power[i]);

    if (type == 'M') // town
      power[i] = -power[i];

    if (type == 'E' && power[i] == 0) { // power station with no production, mark this specially
      capacity[i] = -1;
      parent[i] = 0;
    }

    newpar[i] = parent[i];
    newpow[i] = power[i];
  }

  for (int i=N;i>0;i--) { // reducing the tree
    if (newpow[i] < 0) { // town, just a town
      if (!parent[i]) // no parent
	fail();

      // trying to join the town with its parent
      if (capacity[i] + power[i] < 0) // the town's consumption is more than the uplink capacity
	fail(); // it is impossible to solve

      if (capacity[parent[i]] == -1 // the parent is a power station that has been exhausted
	  || (power[parent[i]] > 0 && power[parent[i]] + power[i] < 0)) // the parent is a too weak power station
	fail(); // it is impossible to solve

      if ((power[parent[i]] += power[i]) < -MAXPOW) // delegate the power one level higher
	fail();
      newpow[parent[i]] += power[i]; // reduce also the virtual power station

      if (power[parent[i]] == 0) // the uplink power station is now exhausted
	capacity[parent[i]] = -1; // the special mark for exhausted power stations

    } else if (newpow[i] > 0) { // power station, induced or real
      if (!parent[i]) // no parent
	continue;

      if (power[parent[i]] > 0 || capacity[parent[i]] == -1) // parent is a real power station
	continue;
      
      int plus = (capacity[i] < newpow[i] ? capacity[i] : newpow[i]) + power[parent[i]]; // the lesser ... what is offered to the parent ... with counting its own consumption

      if (plus > newpow[parent[i]]) { // our offer is the best known for the parent
	newpow[parent[i]] = plus; // 
	newpar[parent[i]] = i;
      }
      // otherwise just let it fly
    }
    // if newpow[i] == 0, nothing is needed to be done ... it is shull null
  }

  for (int i=1;i<=N;i++) // resolving the parts of the net
    findpst(i);

  for (int i=1;i<=N;i++) // output
    printf("%d%c", newpar[i], i == N ? '\n' : ' ');
}

int findpst(int i) {
  assert(i);
  if (power[i] > 0 || capacity[i] == -1)
    return newpar[i] = i;
  
  return newpar[i] = findpst(newpar[i]);
}
